@@ -3,26 +3,37 @@
 #' R functions for managing Python [conda
 #' environments](https://conda.io/docs/user-guide/tasks/manage-environments.html).
 #'
-#' @param envname Name of conda environment
+#' @param envname The name of, or path to, a conda environment.
 #'
-#' @param conda Path to conda executable (or "auto" to find conda using the
-#'   PATH and other conventional install locations).
+#' @param conda The path to a `conda` executable. Use `"auto"` to allow `reticulate` to
+#'   automatically find an appropriate `conda` binary. See **Finding Conda** for more details.
 #'
-#' @param packages Character vector with package names to install or remove.
+#' @param packages A character vector, indicating package names which should be installed or removed.
 #'
-#' @param pip `TRUE` to use pip (defaults to `FALSE`)
+#' @param pip Boolean; use `pip` when downloading or installing packages? Defaults to `FALSE`.
 #'
 #' @param ... Optional arguments, reserved for future expansion.
 #'
 #' @return `conda_list()` returns a data frame with the names and paths to the
 #'   respective python binaries of available environments. `conda_create()`
-#'   returns the Path to the python binary of the created environment.
+#'   returns the path to the python binary of the created environment.
 #'   `conda_binary()` returns the location of the main conda binary or `NULL`
 #'   if none can be found.
 #'
 #' @name conda-tools
 #'
 #' @importFrom jsonlite fromJSON
+#' 
+#' @section Finding Conda:
+#' 
+#' When `conda = "auto"`, `reticulate` will attempt to automatically find an
+#' Anaconda / Miniconda installation and use that. `reticulate` will search the
+#' following locations:
+#' 
+#' 1. The location specified by the `reticulate.conda_binary` \R option;
+#' 2. The [miniconda_path()] location (if it exists);
+#' 3. The program `PATH`;
+#' 4. A set of pre-defined locations where Conda is typically installed.
 #'
 #' @export
 conda_list <- function(conda = "auto") {
@@ -55,11 +66,22 @@ conda_list <- function(conda = "auto") {
   # strip out anaconda cloud prefix (not valid json)
   if (length(conda_envs) > 0 && grepl("Anaconda Cloud", conda_envs[[1]], fixed = TRUE))
     conda_envs <- conda_envs[-1]
+  
+  # parse conda info
+  info <- fromJSON(conda_envs)
 
   # convert to json
-  conda_envs <- fromJSON(conda_envs)$envs
+  conda_envs <- info$envs
   conda_envs <- Filter(file.exists, conda_envs)
-
+  
+  # normalize and remove duplicates (seems necessary on Windows as Anaconda
+  # may report both short-path and long-path versions of the same environment)
+  conda_envs <- unique(normalizePath(conda_envs))
+  
+  # remove root conda environment as it shouldnnt count as environment and
+  # it is not correct to install packages into it.
+  conda_envs <- conda_envs[!conda_envs == normalizePath(info$root_prefix)]
+  
   # return an empty data.frame when no envs are found
   if (length(conda_envs) == 0L) {
     return(data.frame(
@@ -69,10 +91,6 @@ conda_list <- function(conda = "auto") {
     )
   }
 
-  # normalize and remove duplicates (seems necessary on Windows as Anaconda
-  # may report both short-path and long-path versions of the same environment)
-  conda_envs <- unique(normalizePath(conda_envs))
-
   # build data frame
   name <- character()
   python <- character()
@@ -93,10 +111,24 @@ conda_list <- function(conda = "auto") {
 }
 
 
-
+#' @param forge Boolean; include the [Conda Forge](https://conda-forge.org/)
+#'   repository?
+#'
+#' @param channel An optional character vector of Conda channels to include.
+#'   When specified, the `forge` argument is ignored. If you need to
+#'   specify multiple channels, including the Conda Forge, you can use
+#'   `c("conda-forge", <other channels>)`.
+#'
 #' @rdname conda-tools
+#'
+#' @keywords internal
+#'
 #' @export
-conda_create <- function(envname = NULL, packages = "python", conda = "auto") {
+conda_create <- function(envname = NULL,
+                         packages = "python",
+                         forge = TRUE,
+                         channel = character(),
+                         conda = "auto") {
 
   # resolve conda binary
   conda <- conda_binary(conda)
@@ -106,6 +138,16 @@ conda_create <- function(envname = NULL, packages = "python", conda = "auto") {
 
   # create the environment
   args <- conda_args("create", envname, packages)
+
+  # add user-requested channels
+  channels <- if (length(channel))
+    channel
+  else if (forge)
+    "conda-forge"
+
+  for (ch in channels)
+    args <- c(args, "-c", ch)
+
   result <- system2(conda, shQuote(args))
   if (result != 0L) {
     stop("Error ", result, " occurred creating conda environment ", envname,
@@ -140,12 +182,23 @@ conda_remove <- function(envname, packages = NULL, conda = "auto") {
   }
 }
 
-#' @param forge Include the [Conda Forge](https://conda-forge.org/) repository.
-#' @param pip_ignore_installed Ignore installed versions when using pip. This is `TRUE` by default
-#'   so that specific package versions can be installed even if they are downgrades. The `FALSE`
-#'   option is useful for situations where you don't want a pip install to attempt an overwrite
-#'   of a conda binary package (e.g. SciPy on Windows which is very difficult to install via
-#'   pip due to compilation requirements).
+#' @param forge Boolean; include the [Conda Forge](https://conda-forge.org/)
+#'   repository?
+#'   
+#' @param channel An optional character vector of Conda channels to include.
+#'   When specified, the `forge` argument is ignored. If you need to
+#'   specify multiple channels, including the Conda Forge, you can use
+#'   `c("conda-forge", <other channels>)`.
+#'
+#' @param pip_ignore_installed Ignore installed versions when using pip. This is
+#'   `TRUE` by default so that specific package versions can be installed even
+#'   if they are downgrades. The `FALSE` option is useful for situations where
+#'   you don't want a pip install to attempt an overwrite of a conda binary
+#'   package (e.g. SciPy on Windows which is very difficult to install via pip
+#'   due to compilation requirements).
+#'   
+#' @param pip_options An optional character vector of additional command line
+#'   arguments to be passed to `pip`. Only relevant when `pip = TRUE`.
 #'
 #' @rdname conda-tools
 #'
@@ -155,33 +208,45 @@ conda_remove <- function(envname, packages = NULL, conda = "auto") {
 conda_install <- function(envname = NULL,
                           packages,
                           forge = TRUE,
+                          channel = character(),
                           pip = FALSE,
+                          pip_options = character(),
                           pip_ignore_installed = FALSE,
                           conda = "auto",
                           python_version = NULL,
                           ...)
 {
+  check_forbidden_install("Python packages")
+  
   # resolve conda binary
   conda <- conda_binary(conda)
 
   # resolve environment name
   envname <- condaenv_resolve(envname)
 
-  # honor request for specific Python
-  python_package <- "python"
-  if (!is.null(python_version))
-    python_package <- paste(python_package, python_version, sep = "=")
+  # honor request for specific version of Python package
+  python_package <- if (is.null(python_version))
+    "python"
+  else
+    sprintf("python=%s", python_version)
 
-  # check if the environment exists, and create it on demand if needed.
-  # if the environment does already exist, but a version of Python was
-  # requested, attempt to install that in the existing environment
-  # (effectively re-creating it if the Python version differs)
-  python <- tryCatch(conda_python(envname = envname, conda = conda), error = identity)  
+  # check to see if we already have a valid Python installation for
+  # this conda environment
+  python <- tryCatch(
+    conda_python(envname = envname, conda = conda),
+    error = identity
+  )
   
+  # if this conda environment doesn't seem to exist, auto-create it
   if (inherits(python, "error") || !file.exists(python)) {
     conda_create(envname, packages = python_package, conda = conda)
     python <- conda_python(envname = envname, conda = conda)
-  } else if (!is.null(python_package)) {
+  }
+  
+  # if the user has requested a specific version of Python, ensure that
+  # version of Python is installed into the requested environment
+  # (should be no-op if that copy of Python already installed)
+  if (!is.null(python_version)) {
     args <- conda_args("install", envname, python_package)
     status <- system2(conda, shQuote(args))
     if (status != 0L) {
@@ -192,13 +257,31 @@ conda_install <- function(envname = NULL,
   }
 
   # delegate to pip if requested
-  if (pip)
-    return(pip_install(python, packages))
+  if (pip) {
     
+    result <- pip_install(
+      python = python,
+      packages = packages,
+      pip_options = pip_options,
+      ignore_installed = pip_ignore_installed
+    )
+    
+    return(result)
+    
+  }
+  
   # otherwise, use conda
   args <- conda_args("install", envname)
-  if (forge)
-    args <- c(args, "-c", "conda-forge")
+  
+  # add user-requested channels
+  channels <- if (length(channel))
+    channel
+  else if (forge)
+    "conda-forge"
+  
+  for (ch in channels)
+    args <- c(args, "-c", ch)
+    
   args <- c(args, python_package, packages)
   result <- system2(conda, shQuote(args))
   
@@ -225,6 +308,8 @@ conda_binary <- function(conda = "auto") {
     conda <- conda[[1]]
   }
 
+  conda <- normalizePath(conda, winslash = "/", mustWork = FALSE)
+  
   # if the user has requested a conda binary in the 'condabin' folder,
   # try to find and use its sibling in the 'bin' folder instead as
   # we rely on other tools typically bundled in the 'bin' folder
@@ -288,56 +373,83 @@ conda_python <- function(envname = NULL, conda = "auto") {
 
 
 find_conda <- function() {
+  
+  # allow specification of conda executable
+  conda <- getOption("reticulate.conda_binary")
+  if (!is.null(conda))
+    return(conda)
+  
+  # if miniconda is installed, use it
+  if (miniconda_exists())
+    return(miniconda_conda())
+  
+  # if there is a conda executable on the PATH, use it
   conda <- Sys.which("conda")
-  if (!nzchar(conda)) {
-    conda_locations <- c(
-      miniconda_conda(),
-      path.expand("~/opt/anaconda/bin/conda"),
-      path.expand("~/opt/anaconda2/bin/conda"),
-      path.expand("~/opt/anaconda3/bin/conda"),
-      path.expand("~/opt/anaconda4/bin/conda"),
-      path.expand("~/opt/miniconda/bin/conda"),
-      path.expand("~/opt/miniconda2/bin/conda"),
-      path.expand("~/opt/miniconda3/bin/conda"),
-      path.expand("~/opt/miniconda4/bin/conda"),
-      path.expand("~/anaconda/bin/conda"),
-      path.expand("~/anaconda2/bin/conda"),
-      path.expand("~/anaconda3/bin/conda"),
-      path.expand("~/anaconda4/bin/conda"),
-      path.expand("~/miniconda/bin/conda"),
-      path.expand("~/miniconda2/bin/conda"),
-      path.expand("~/miniconda3/bin/conda"),
-      path.expand("~/miniconda4/bin/conda"),
-      path.expand("/anaconda/bin/conda"),
-      path.expand("/anaconda2/bin/conda"),
-      path.expand("/anaconda3/bin/conda"),
-      path.expand("/anaconda4/bin/conda"),
-      path.expand("/miniconda/bin/conda"),
-      path.expand("/miniconda2/bin/conda"),
-      path.expand("/miniconda3/bin/conda"),
-      path.expand("/miniconda4/bin/conda")
-    )
-    if (is_windows()) {
-      anaconda_versions <- windows_registry_anaconda_versions()
-      anaconda_versions <- subset(anaconda_versions, anaconda_versions$arch == .Platform$r_arch)
-      if (nrow(anaconda_versions) > 0) {
-        conda_scripts <- utils::shortPathName(
-          file.path(anaconda_versions$install_path, "Scripts", "conda.exe")
-        )
-        conda_bats <- utils::shortPathName(
-          file.path(anaconda_versions$install_path, "condabin", "conda.bat")
-        )
-        conda_locations <- c(conda_locations, conda_bats, conda_scripts)
-      }
+  if (nzchar(conda))
+    return(conda)
+  
+  # otherwise, search common locations for conda
+  # TODO: these won't work on Windows as conda is often installed
+  # at e.g. 'condabin/conda.bat' or similar!
+  conda_locations <- c(
+    miniconda_conda(),
+    path.expand("~/opt/anaconda/bin/conda"),
+    path.expand("~/opt/anaconda2/bin/conda"),
+    path.expand("~/opt/anaconda3/bin/conda"),
+    path.expand("~/opt/anaconda4/bin/conda"),
+    path.expand("~/opt/miniconda/bin/conda"),
+    path.expand("~/opt/miniconda2/bin/conda"),
+    path.expand("~/opt/miniconda3/bin/conda"),
+    path.expand("~/opt/miniconda4/bin/conda"),
+    path.expand("~/anaconda/bin/conda"),
+    path.expand("~/anaconda2/bin/conda"),
+    path.expand("~/anaconda3/bin/conda"),
+    path.expand("~/anaconda4/bin/conda"),
+    path.expand("~/miniconda/bin/conda"),
+    path.expand("~/miniconda2/bin/conda"),
+    path.expand("~/miniconda3/bin/conda"),
+    path.expand("~/miniconda4/bin/conda"),
+    path.expand("/opt/anaconda/bin/conda"),
+    path.expand("/opt/anaconda2/bin/conda"),
+    path.expand("/opt/anaconda3/bin/conda"),
+    path.expand("/opt/anaconda4/bin/conda"),
+    path.expand("/opt/miniconda/bin/conda"),
+    path.expand("/opt/miniconda2/bin/conda"),
+    path.expand("/opt/miniconda3/bin/conda"),
+    path.expand("/opt/miniconda4/bin/conda"),
+    path.expand("/anaconda/bin/conda"),
+    path.expand("/anaconda2/bin/conda"),
+    path.expand("/anaconda3/bin/conda"),
+    path.expand("/anaconda4/bin/conda"),
+    path.expand("/miniconda/bin/conda"),
+    path.expand("/miniconda2/bin/conda"),
+    path.expand("/miniconda3/bin/conda"),
+    path.expand("/miniconda4/bin/conda")
+  )
+  
+  # on Windows, check the registry for a compatible version of Anaconda
+  if (is_windows()) {
+    anaconda_versions <- windows_registry_anaconda_versions()
+    anaconda_versions <- subset(anaconda_versions, anaconda_versions$arch == .Platform$r_arch)
+    if (nrow(anaconda_versions) > 0) {
+      conda_scripts <- utils::shortPathName(
+        file.path(anaconda_versions$install_path, "Scripts", "conda.exe")
+      )
+      conda_bats <- utils::shortPathName(
+        file.path(anaconda_versions$install_path, "condabin", "conda.bat")
+      )
+      conda_locations <- c(conda_locations, conda_bats, conda_scripts)
     }
-    conda_locations <- conda_locations[file.exists(conda_locations)]
-    if (length(conda_locations) > 0)
-      conda_locations
-    else
-      NULL
-  } else {
-    conda
   }
+  
+  # keep only conda locations that exist
+  conda_locations <- conda_locations[file.exists(conda_locations)]
+  if (length(conda_locations))
+    return(conda_locations)
+  
+  # explicitly return NULL when no conda found
+  NULL
+  
 }
 
 condaenv_resolve <- function(envname = NULL) {
@@ -416,7 +528,16 @@ conda_list_packages <- function(envname = NULL, conda = "auto", no_pip = TRUE) {
     package     = parsed$name,
     version     = parsed$version,
     requirement = paste(parsed$name, parsed$version, sep = "="),
+    channel     = parsed$channel,
     stringsAsFactors = FALSE
   )
   
 }
+
+conda_installed <- function() {
+  condabin <- tryCatch(conda_binary(), error = identity)
+  if (inherits(condabin, "error"))
+    return(FALSE)
+  else
+    return(TRUE)
+}
